unit uTaskProgress;

interface

uses
  Classes, Windows, Messages, UxTheme, SysUtils, CommCtrl,
  {$IF CompilerVersion >= 23.0}
  Vcl.Controls, Vcl.Dialogs, Vcl.ComCtrls, Vcl.Forms,
  {$ELSE}
  Controls, Dialogs, ComCtrls, Forms,
  {$IFEND}
  uWindowsVersion;

type
  {$IF CompilerVersion < 20.0}
  TProgressBarStyle = (pbstNormal, pbstMarquee);
  {$IFEND}

  TProgressBarEx = class(TProgressBar)
  {$IF CompilerVersion < 20.0}
  private
    FMarqueeInterval: Integer;
    FStyle: TProgressBarStyle;
    function GetState: TProgressBarState;
    procedure SetMarqueeInterval(const Value: Integer);
    procedure SetState(const Value: TProgressBarState);
    procedure SetStyle(const Value: TProgressBarStyle);
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Style: TProgressBarStyle read Fstyle write SetStyle default pbstNormal;
    property MarqueeInterval: Integer read FMarqueeInterval write SetMarqueeInterval default 10;
    property State: TProgressBarState read GetState write SetState default pbsNormal;
  {$IFEND}
  end;

  TClientType = (ctNative, ctNonNative);

  TTaskDialogProgress = class(TPersistent)
  private
    FClientNative: HWND;
    FClientNonNative: TProgressbarEx;
    FClientType: TClientType;
    FMarqueeSpeed: Cardinal;
    FMax: Integer;
    FMin: Integer;
    FPosition: Integer;
    FState: TProgressBarState;
    FIsMarquee: Boolean;
    procedure SetMarqueeSpeed(const Value: Cardinal);
    procedure SetMax(const Value: Integer);
    procedure SetMin(const Value: Integer);
    procedure SetPosition(const Value: Integer);
    procedure SetState(const Value: TProgressBarState);
    procedure SetClientNative(const Value: HWND);
    procedure SetClientNonNative(const Value: TProgressbarEx);
  public
    constructor Create;
    procedure Initialize;
    property ClientNative: HWND read FClientNative write SetClientNative;
    property ClientNonNative: TProgressbarEx read FClientNonNative write SetClientNonNative;
  published
    property IsMarquee: Boolean read FIsMarquee write FIsMarquee;
    property MarqueeSpeed: Cardinal read FMarqueeSpeed write SetMarqueeSpeed default 0;
    property Max: Integer read FMax write SetMax default 100;
    property Min: Integer read FMin write SetMin default 0;
    property Position: Integer read FPosition write SetPosition default 0;
    property State: TProgressBarState read FState write SetState default pbsNormal;
  end;

implementation

{ TTaskDialogProgress }

const
  ProgressBarStates: array[TProgressBarState] of Integer = (PBST_NORMAL, PBST_ERROR, PBST_PAUSED);

constructor TTaskDialogProgress.Create;
begin
  inherited Create;
  FClientNative := 0;
  FClientNonNative := nil;
  FClientType := ctNative;
  FMax := 100;
  FMin := 0;
  FPosition := 0;
  FMarqueeSpeed := 0;
  FState := pbsNormal;
  FIsMarquee := False;
end;

procedure TTaskDialogProgress.Initialize;
begin
  if FClientType = ctNative then
    begin
      SendMessage(FClientNative, TDM_SET_MARQUEE_PROGRESS_BAR, Windows.WPARAM(BOOL(FIsMarquee)), 0);
      if FIsMarquee then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_MARQUEE, Windows.WPARAM(BOOL(True)), FMarqueeSpeed)
      else
        begin
          SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_RANGE, 0, MakeLParam(FMin, FMax));
          SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_POS, FPosition, 0);
        end;
      SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_STATE, ProgressBarStates[FState], 0);
    end
  else
    begin
      if FIsMarquee then
        begin
          FClientNonNative.Style := pbstMarquee;
          FClientNonNative.MarqueeInterval := FMarqueeSpeed;
        end
      else
        begin
          FClientNonNative.Min := FMin;
          FClientNonNative.Max := FMax;
          FClientNonNative.Position := FPosition;
        end;
      FClientNonNative.State := FState;
    end;
end;

procedure TTaskDialogProgress.SetClientNative(const Value: HWND);
begin
  FClientNative := Value;
  FClientType := ctNative;
  Self.Initialize;
end;

procedure TTaskDialogProgress.SetClientNonNative(const Value: TProgressbarEx);
begin
  FClientNonNative := Value;
  FClientType := ctNonNative;
  Self.Initialize;
end;

procedure TTaskDialogProgress.SetMarqueeSpeed(const Value: Cardinal);
begin
  if Value <> FMarqueeSpeed then
    begin
      FMarqueeSpeed := Value;
      if FClientType = ctNative then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_MARQUEE, Windows.WPARAM(BOOL(True)), FMarqueeSpeed)
      else
        FClientNonNative.MarqueeInterval := FMarqueeSpeed;
    end;
end;

procedure TTaskDialogProgress.SetMax(const Value: Integer);
begin
  if Value <> FMax then
    begin
      FMax := Value;
      if FClientType = ctNative then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_RANGE, 0, MakeLParam(FMin, FMax))
      else
        FClientNonNative.Max := FMax;
    end;
end;

procedure TTaskDialogProgress.SetMin(const Value: Integer);
begin
  if Value <> FMin then
    begin
      FMin := Value;
      if FClientType = ctNative then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_RANGE, 0, MakeLParam(FMin, FMax))
      else
        FClientNonNative.Min := FMin;
    end;
end;

procedure TTaskDialogProgress.SetPosition(const Value: Integer);
begin
  if Value <> FPosition then
    begin
      FPosition := Value;
      if FClientType = ctNative then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_POS, FPosition, 0)
      else
        FClientNonNative.Position := FPosition;
    end;
end;

procedure TTaskDialogProgress.SetState(const Value: TProgressBarState);
begin
  if Value <> FState then
    begin
      FState := Value;
      if VistaOrLater then
        SendMessage(FClientNative, TDM_SET_PROGRESS_BAR_STATE, ProgressBarStates[FState], 0);
    end;
end;

{ TProgressBarEx }

{$IF CompilerVersion < 20.0}
constructor TProgressBarEx.Create(AOwner: TComponent);
begin
  inherited;
  FStyle := pbstNormal;
  FMarqueeInterval := 10;
end;

procedure TProgressBarEx.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if FStyle = pbstMarquee then
    with Params do
      Style := Style or PBS_MARQUEE;
end;

function TProgressBarEx.GetState: TProgressBarState;
begin
  if VistaOrLater then
    result := TProgressBarState(SendMessage(Self.Handle, PBM_GETSTATE, 0, 0))
  else
    result := pbsNormal;
end;

procedure TProgressBarEx.SetMarqueeInterval(const Value: Integer);
begin
  FMarqueeInterval := Value;
  SendMessage(Self.Handle, PBM_SETMARQUEE, WPARAM(THandle(Boolean(FStyle))), LPARAM(Value));
end;

procedure TProgressBarEx.SetState(const Value: TProgressBarState);
begin
  if VistaOrLater then
    SendMessage(Self.Handle, PBM_SETSTATE, WPARAM(Value), 0);
end;

procedure TProgressBarEx.SetStyle(const Value: TProgressBarStyle);
begin
  FStyle := Value;
  SendMessage(Self.Handle, PBM_SETMARQUEE, WPARAM(THandle(Boolean(Value))), LPARAM(FMarqueeInterval));
end;
{$IFEND}

end.
